package nrs.jester;

import java.sql.Types;

/*
 * Java Stored Procedure Tester (Jester)
 * Copyright (C) 2002 Nick Sydenham <nsydenham@yahoo.co.uk>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 */

/**
 * Definition of a stored procedure parameter
 *
 * @author Nicholas Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class Parameter
{
    /** Constants for IN and OUT parameters */
    public final static int IN = 0;
    public final static int OUT = 1;
    
    private int inout = IN;
    private int dataType = Types.VARCHAR;
    private String description = null;
    private String tip = null;
    private int length = 20;
    private boolean nullable = false;
    
    public Parameter(int inoutType, int dType, String desc)
    {
	inout = inoutType;
	dataType = dType;
	description = desc;
    }
    
    /**
     * Get the maximum length of the field
     * @return int
     */
    public int getLength()
    {
	return length;
    }
    
    /**
     * Get a description of the parameter
     * @return description
     */
    public String getDescription()
    {
	return description;
    }
    
    /**
     * Get a tip to display when the mouse hovers over the parameter
     * @return the parameter tip
     */
    public String getTip()
    {
	return tip;
    }
    
    /**
     * Get the parameter data type
     * @return an integer representing the type
     */
    public int getDataType()
    {
	return dataType;
    }
    
    /**
     * Is this an input or output parameter
     * @return int
     */
    public int getInOut() {
	return inout;
    }
    
    /**
     * Set the length of the parameter
     * @param newLength int
     */
    public void setLength(int newLength)
    {
	length = newLength;
    }
    
    /**
     * Set the description of the parameter
     * @param newDescription description of the parameter
     */
    public void setDescription(String newDescription)
    {
	description = newDescription;
    }
    
    /**
     * Set the tip to display when the mouse hovers over the parameter
     * @param newTip new tip
     */
    public void setTip(String newTip)
    {
	tip = newTip;
    }
    
    /**
     * Set the data type of the parameter
     * @param newType from java.sql.Types
     */
    public void setDataType(int newDataType)
    {
	dataType = newDataType;
    }
    
    /**
     * Set whether this parameter can be null. This doesn't apply to 
     * output parameters.
     * @param nullable true to indicate the parameter can be null
     */
    public void setNullable(boolean nullable) {
	this.nullable = nullable;
    }
    
    /**
     * Can this parameter be null?
     * @return true if the parameter can be null
     */
    public boolean isNullable() {
	return nullable;
    }
    
    /**
     * Get the parameter details as an XML String
     * @return an XML String representing this parameter
     */
    public String getDetails()
    {
	StringBuffer sb = new StringBuffer("<");
	sb.append(getElementName());
	sb.append(" datatype='");
	sb.append(dataType);
	sb.append("' length='");
	sb.append(length);
	if (inout == IN) {
	    sb.append("' nullable='");
	    sb.append(nullable);
	    if (tip != null) {
		sb.append("' tip='");
		sb.append(tip);
	    }
	}
	sb.append("'>");
	sb.append(description);
	sb.append("</");
	sb.append(getElementName());
	sb.append(">\n");
	
	return sb.toString();
    }
    
    /**
     * Get the element name based on the inout type
     * @return either in or out appropriately
     */
    private String getElementName()
    {
	switch (inout) {
	    case IN: return "in";
		
	    case OUT: return "out";
	}
	
	return null;
    }
}
