package nrs.jester;

import java.awt.*;
import java.awt.event.*;
import java.sql.*;
import java.util.*;
import javax.swing.*;

/*
 * Java Stored Procedure Tester (Jester)
 * Copyright (C) 2002 Nick Sydenham <nsydenham@yahoo.co.uk>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 */

/**
 * Panel providing inputs for a stored procedure
 *
 * @author Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class SPInputPanel extends JPanel
{
    private Jester jester = null;
    private StoredProcedure sp = null;
    private JTextField[] tfValues;
    private java.util.List inParams = null;
    
    /** Creates a new instance of SPInputPanel */
    public SPInputPanel(Jester jester, StoredProcedure sp)
    {
	JTextField tf;
        String defVal;
        inParams = sp.getInParameters();
        String tip;
	String description;
	
	this.jester = jester;
	this.sp = sp;
        
        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        setLayout(gridbag);
        
	description = sp.getDescription();
        if (description != null && description.length() > 0) {
            c.gridwidth = GridBagConstraints.REMAINDER;
	    c.weightx = 1.0;
            c.insets = new Insets(5, 5, 10, 5);
            c.anchor = GridBagConstraints.CENTER;
	    c.fill = GridBagConstraints.HORIZONTAL;
            JLabel desc = new JLabel(sp.getDescription());
            gridbag.setConstraints(desc, c);
            add(desc);
        }
        
        Iterator it = inParams.iterator();
        Parameter param;
        tfValues = new JTextField[inParams.size()];
        int i = 0;
	c.anchor = GridBagConstraints.WEST;
	c.weighty = 0.0;
        while (it.hasNext()) {
            param = (Parameter)it.next();
            c.gridwidth = 1;
	    c.fill = GridBagConstraints.NONE;
	    c.weightx = 0.0;
            c.insets = new Insets(0, 10, 5, 20);
            add(jester.makeLabel(param.getDescription(), gridbag, c));
            
            tf = new JTextField(15);
            tf.setColumns(param.getLength());
            // set any tool tip
            tip = param.getTip();
            if (tip == null) {
		if (param.getDataType() == Types.DATE) tip = "Format: "+jester.getDateFormat();
		else if (param.getDataType() == Types.TIME) tip = "Format: "+jester.getTimeFormat();
	    }
	    tf.setToolTipText(tip);
            
            // see if there is a default value for the field
            defVal = jester.getDefaultParameterValue(param.getDescription());
            if (defVal != null) tf.setText(defVal);
            tfValues[i] = tf;
            c.gridwidth = GridBagConstraints.REMAINDER;
	    c.fill = GridBagConstraints.HORIZONTAL;
	    c.weightx = 1.0;
            c.insets = new Insets(0, 0, 5, 10);
            gridbag.setConstraints(tf, c);
            add(tfValues[i++]);
        }
        
	JPanel buttonPanel = new JPanel();
	buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
	JButton editButton = new JButton("Edit");
	editButton.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		editSP();
	    }
	});
	buttonPanel.add(editButton);
	buttonPanel.add(Box.createRigidArea(new Dimension(5, 0)));
	
	JButton deleteButton = new JButton("Delete");
	deleteButton.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		deleteSP();
	    }
	});
	buttonPanel.add(deleteButton);
	
	buttonPanel.add(Box.createHorizontalGlue());
	
        JButton callButton = new JButton("Call", new ImageIcon(jester.getImage(Jester.GO_ICON)));
	callButton.setToolTipText("Click to execute stored procedure");
        callButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                callSP();
            }
        });
	buttonPanel.add(callButton);

        c.anchor = GridBagConstraints.SOUTH;
        c.weighty = 1.0;
	c.weightx = 1.0;
	c.insets = new Insets(15, 10, 10, 10);
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.fill = GridBagConstraints.HORIZONTAL;
        gridbag.setConstraints(buttonPanel, c);
        add(buttonPanel);
    }
    
    /**
     * Call the StoredProcedure
     */
    private void callSP()
    {
	String[] values = new String[tfValues.length];
	String paramVal;
	Parameter param;
	int i;
	
	for (i=0; i<tfValues.length; i++) {
	    paramVal = tfValues[i].getText();
	    if (paramVal.length() == 0) paramVal = null;
	    values[i] = paramVal;
	}
	
	// perform some input checks
	i = 0;
	Iterator it = inParams.iterator();
	while (it.hasNext()) {
	    param = (Parameter)it.next();
	    paramVal = values[i];
	    
	    // check for null
	    if (!param.isNullable() && paramVal == null) {
		jester.setStatus("Input parameter "+param.getDescription()+
		    " cannot be null.");
		tfValues[i].requestFocus();
		return;
	    }

	    // see if it is too large
	    if (paramVal != null && (paramVal.length() > param.getLength()))
	    {
		switch (param.getDataType()) {
		    case Types.CHAR:
		    case Types.VARCHAR:
		    case Types.LONGVARCHAR:
			jester.setStatus("Input parameter "+param.getDescription()+
			    " is too large! Maximum number of characters is "+param.getLength());
			tfValues[i].requestFocus();
			return;
		}
	    }
	    
	    i++;
	}
	
	jester.setCursor(new Cursor(Cursor.WAIT_CURSOR));
	jester.callSP(sp, values);
	jester.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
    }
    
    private void editSP() {
	jester.editSP();
    }
    
    private void deleteSP() {
	jester.deleteSP();
    }
    
    /** 
     * Set the value of an input parameter
     * @param value the value
     * @param the description of the parameter
     */
    public void setInputValue(String value, String paramDescription)
    {
	Parameter param;
	int i = 0;
	java.util.List l = sp.getInParameters();
	Iterator it = l.iterator();
	while (it.hasNext()) {
	    param = (Parameter)it.next();
	    if (paramDescription.equals(param.getDescription())) {
		tfValues[i].setText(value);
		break;
	    }
	    i++;
	}
    }
}
