package nrs.jester;

import javax.swing.event.*;
import java.io.*;
import java.util.*;
import javax.swing.ListModel;
import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;

/*
 * Java Stored Procedure Tester (Jester)
 * Copyright (C) 2002 Nick Sydenham <nsydenham@yahoo.co.uk>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 */

/**
 * Reads the stored procedure XML file and is the ListModel for the list
 *
 * @author Nicholas Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class SPReader extends DefaultHandler implements ListModel
{
    private String currentElement;
    private CharArrayWriter contents = new CharArrayWriter();
    private int paramType = -1;
    private String paramDescription=null;
    private String tip=null, length=null, nullable=null;
    private StoredProcedure sp;
    private List storedProcedures = new ArrayList();
    private List listeners = new ArrayList();
    
    /**
     * Add a listener to the list that's notified each time a change
     * to the data model occurs.
     * @param l the ListDataListener
     */
    public void addListDataListener(ListDataListener l)
    {
	listeners.add(l);
    }
    
    /**
     * Receive notification of character data inside an element.
     * @param ch The characters.
     * @param start The start position in the character array.
     * @param length The number of characters to use from the
     *               character array.
     * @exception org.xml.sax.SAXException Any SAX exception, possibly
     *            wrapping another exception.
     */
    public void characters(char[] ch, int start, int length) throws SAXException
    {
	contents.write(ch, start, length);
    }
    
    /**
     * Dump the StoredProcedure definitions to stdout
     */
    public void dump()
    {
	StoredProcedure sp;
	
	Iterator it = storedProcedures.iterator();
	
	while (it.hasNext())
	{
	    sp = (StoredProcedure)it.next();
	    System.out.println(sp.getDetails());
	}
    }
    
    /** Start of the document */
    public void startDocument() throws SAXException
    {
	storedProcedures = new ArrayList();
    }
    
    /** End of the document */
    public void endDocument() throws SAXException
    {
	// tell the list listeners that the data has changed
	ListDataListener l;
	ListDataEvent lde = new ListDataEvent(this, ListDataEvent.CONTENTS_CHANGED, 0, storedProcedures.size());
	
	Iterator it = listeners.iterator();
	while (it.hasNext())
	{
	    l = (ListDataListener)it.next();
	    l.contentsChanged(lde);
	}
    }
    
    /**
     * Receive notification of the end of an element.
     *
     * @param name The element type name.
     * @param attributes The specified or defaulted attributes.
     * @exception org.xml.sax.SAXException Any SAX exception, possibly
     *            wrapping another exception.
     */
    public void endElement(String uri, String localName, String qName) throws SAXException
    {
	if (localName.equalsIgnoreCase("StoredProcedure"))
	    storedProcedures.add(sp);
	
	else if (localName.equalsIgnoreCase("in"))
	{
	    Parameter param = sp.addInParameter(paramType, tip, contents.toString());
	    if (length != null)
	    {
	        try
		{
		    int pl = Integer.parseInt(length);
		    param.setLength(pl);
		} catch (NumberFormatException nfex) {
		    throw new SAXException("Invalid size attribute on sp "+sp.getName());
		}
	    }
	    if (nullable != null && nullable.equalsIgnoreCase("true"))
		param.setNullable(true);
	}
	
	else if (localName.equalsIgnoreCase("out"))
	    sp.addOutParameter(paramType, contents.toString());
	
	else if (localName.equalsIgnoreCase("description"))
	    sp.setDescription(contents.toString());
	
	paramType = -1;
	tip = null;
    }
    
    /**
     * Returns the value at the specified index.
     */
    public Object getElementAt(int index)
    {
	return storedProcedures.get(index);
    }
    
    /**
     * Returns the length of the list.
     */
    public int getSize()
    {
	return storedProcedures.size();
    }
    
    /**
     * Remove a listener from the list that's notified each time a
     * change to the data model occurs.
     * @param l the ListDataListener
     */
    public void removeListDataListener(javax.swing.event.ListDataListener l)
    {
	listeners.remove(l);
    }
    
    /**
     * Receive notification of the start of an element.
     *
     * @param name The element type name.
     * @param attributes The specified or defaulted attributes.
     * @exception org.xml.sax.SAXException Any SAX exception, possibly
     *            wrapping another exception.
     */
    public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException
    {
	String name, pt;
	
	currentElement = localName;
	length = null;
	tip = null;
	nullable = null;
	
	if (localName.equalsIgnoreCase("StoredProcedure"))
	{
	    name = attributes.getValue("name");
	    if (name == null) throw new SAXException("No name attribute on the StoredProcedure element");
	    
	    sp = new StoredProcedure(name);
	}
	else if (localName.equalsIgnoreCase("in") || localName.equalsIgnoreCase("out"))
	{
	    pt = attributes.getValue("datatype");
	    if (pt == null) throw new SAXException("No datatype attribute on parameter");
	    try {
		paramType = Integer.parseInt(pt);
	    } catch (NumberFormatException nfex) {
		throw new SAXException("Incorrect parameter type: "+pt);
	    }
	    
	    tip = attributes.getValue("tip");
	    if (tip != null) tip = tip.trim();
	    length = attributes.getValue("length");
	    nullable = attributes.getValue("nullable");
	    
	    paramDescription = null;
	}
	
	contents.reset();
    }
    
    /**
     * Get the list of stored procedures
     */
    public List getStoredProcedures() {
	return storedProcedures;
    }
    
    /**
     * Set the stored procedures
     * @param newSPList List of StoredProcedure objects
     */
    public void setStoredProcedures(List newSPList) {
	storedProcedures = newSPList;
	// tell listeners that the data has changed
	try { endDocument(); } catch (SAXException sex) {}
    }
    
    /**
     * Remove the specified stored procedure from the list
     * @param sp StoredProcedure to remove
     */
    public void removeStoredProcedure(StoredProcedure sp) {
	int index = storedProcedures.indexOf(sp);
	if (index > -1) {
	    storedProcedures.remove(sp);
	    ListDataEvent lde = new ListDataEvent(this, ListDataEvent.INTERVAL_REMOVED, index, index);
	    
	    ListDataListener l;
	    Iterator it = listeners.iterator();
	    while (it.hasNext())
	    {
		l = (ListDataListener)it.next();
		l.intervalRemoved(lde);
	    }
	}
    }
}
