package nrs.jester;

import java.sql.Types;
import java.util.*;

/*
 * Java Stored Procedure Tester (Jester)
 * Copyright (C) 2002 Nick Sydenham <nsydenham@yahoo.co.uk>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA
 */

/**
 * Description of a stored procedure
 *
 * @author Nicholas Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class StoredProcedure
{
    private String name = null;
    private String description = null;
    private List inParameters = new ArrayList();
    private List outParameters = new ArrayList();
    
    /**
     * StoredProcedure constructor comment.
     */
    public StoredProcedure()
    {
        super();
    }
    
    /**
     * Create a new StoredProcedure with the specified name
     * @param name name of the SP
     */
    public StoredProcedure(String name)
    {
        this.name = name;
    }
    
    /**
     * Add the definition of an input parameter
     * @param dataType the type of the parameter
     * @param tip the parameter tip
     * @param description a description of the parameter
     * @return the parameter
     */
    public Parameter addInParameter(int dataType, String tip, String description) {
        Parameter param = new Parameter(Parameter.IN, dataType, description);
        if (tip != null && tip.length() > 0) param.setTip(tip);
        
        inParameters.add(param);
        
        return param;
    }
    
    /**
     * Add the definition of an output parameter
     * @param dataType the type of the parameter
     * @param description a description of the parameter
     * @return the parameter
     */
    public Parameter addOutParameter(int dataType, String description) {
        Parameter param = new Parameter(Parameter.OUT, dataType, description);
        outParameters.add(param);
        
        return param;
    }
    
    /**
     * Get the description
     * @return description
     */
    public String getDescription() {
        return description;
    }
    
    /**
     * Get a list of input parameters
     * @return java.util.List
     */
    public List getInParameters() {
        return inParameters;
    }
    
    /**
     * Get the Stored Procedure name
     * @return name of the stored procedure
     */
    public String getName() {
        return name;
    }
    
    /**
     * Get a list of ouput parameters
     * @return java.util.List
     */
    public List getOutParameters() {
        return outParameters;
    }
    
    /**
     * Get the details as a string
     * @return an XML String representing the SP
     */
    public String getDetails()
    {
        Parameter param;
	StringBuffer sb = new StringBuffer("	<StoredProcedure name='");
	sb.append(name);
	sb.append("'>\n");
        
	if (description != null) {
	    sb.append("	    <description>");
	    sb.append(description);
	    sb.append("</description>\n");
	}
	
        Iterator it = inParameters.iterator();
        while (it.hasNext()) {
            param = (Parameter)it.next();
	    sb.append("	    ");
	    sb.append(param.getDetails());
        }
        
        it = outParameters.iterator();
        while (it.hasNext()) {
            param = (Parameter)it.next();
	    sb.append("	    ");
	    sb.append(param.getDetails());
        }
	sb.append("	</StoredProcedure>\n");
	
	return sb.toString();
    }
    
    /**
     * Set the description
     * @param newDescription new description
     */
    public void setDescription(String newDescription) {
        description = newDescription;
    }
    
    /**
     * Set the input parameters
     * @param newInParameters list of input parameters
     */
    public void setInParameters(List newInParameters) {
        inParameters = newInParameters;
    }
    
    /**
     * Set the name of the stored procedure
     * @param newName java.lang.String
     */
    public void setName(String newName) {
        name = newName;
    }
    
    /**
     * Set the output parameters
     * @param newOutParameters list of output parameters
     */
    public void setOutParameters(List newOutParameters) {
        outParameters = newOutParameters;
    }
    
    /**
     * Returns the name of the SP
     * @return name of the stored procedure
     */
    public String toString() {
        return name;
    }
}
